==> InfiniTec.Exchange_0.96_Source.zip: WebDav/Message.cs <==
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.CompilerServices;
using System.Net.Mail;
using System.Text.RegularExpressions;

namespace InfiniTec.Exchange.WebDav
{
    /// <summary>
    /// This class provides access to commonly used properties used on messages.
    /// </summary>
    [SchemaClass(typeof(NamedProperties.Message))]
    public class Message : TypedItemBase
    {
        private static string[] _RequiredFields;

        private ChangeNotifyCollection<EmailAddress> _ToRecipients;
        private ChangeNotifyCollection<EmailAddress> _CcRecipients;
        private ChangeNotifyCollection<EmailAddress> _BccRecipients;
        private EmailAddress _SenderAddress;
        private EmailAddress _FromAddress;
        private EmailAddress _ReplyToAddress;
        private bool _PropertiesLoaded;

        /// <summary>
        /// This method is called before the <see cref="M:Refresh"/> or <see cref="M:RefreshAsync"/>
        /// operation is called.
        /// </summary>
        protected override void PreRefresh()
        {
            _ToRecipients = null;
            _CcRecipients = null;
            _BccRecipients = null;
            _SenderAddress = null;
            _FromAddress = null;
            _ReplyToAddress = null;
            _PropertiesLoaded = false;

            base.PreRefresh();
        }

        /// <summary>
        /// This method is called before the <see cref="M:Save"/> or <see cref="M:SaveAsync"/>
        /// operation is called.
        /// </summary>
        protected override void PreSave()
        {
            CommitAddressList(_ToRecipients, NamedProperties.Message.To);
            CommitAddressList(_CcRecipients, NamedProperties.Message.Cc);
            CommitAddressList(_BccRecipients, NamedProperties.Message.Bcc);

            CommitAddress(_FromAddress, NamedProperties.Message.From);
            CommitAddress(_SenderAddress, NamedProperties.Message.Sender);
            CommitAddress(_ReplyToAddress, NamedProperties.Message.ReplyTo);

            base.PreSave();
        }

        private void CommitAddress(EmailAddress address, string propertyName)
        {
            EmailAddress currentValue;

            LoadAddress(out currentValue, propertyName);
            if (currentValue.ToString() == address.ToString()) return;

            SetPropertyValue<string>(propertyName, address.ToString());
        }

        [MethodImpl(MethodImplOptions.Synchronized)]
        private void EnsureAddressProperties()
        {
            if (_PropertiesLoaded) return;

            LoadAddressList(out _ToRecipients, NamedProperties.Message.To);
            LoadAddressList(out _CcRecipients, NamedProperties.Message.Cc);
            LoadAddressList(out _BccRecipients, NamedProperties.Message.Bcc);

            LoadAddress(out _FromAddress, NamedProperties.Message.From);
            LoadAddress(out _SenderAddress, NamedProperties.Message.Sender);
            LoadAddress(out _ReplyToAddress, NamedProperties.Message.ReplyTo);

            _PropertiesLoaded = true;
        }

        /// <summary>
        /// Gets the sender.
        /// </summary>
        /// <value>The sender.</value>
        public EmailAddress Sender
        {
            get
            {
                EnsureAddressProperties();
                return _SenderAddress;
            }

        }
        /// <summary>
        /// Gets the direct recipients.
        /// </summary>
        /// <value>The direct recipients.</value>
        public IList<EmailAddress> ToRecipients
        {
            get
            {
                EnsureAddressProperties();
                return _ToRecipients;
            }
        }
        /// <summary>
        /// Gets the CC recipients.
        /// </summary>
        /// <value>The CC recipients.</value>
        public IList<EmailAddress> CcRecipients
        {
            get
            {
                EnsureAddressProperties();
                return _CcRecipients;
            }
        }
        /// <summary>
        /// Gets the BCC recipients.
        /// </summary>
        /// <value>The BCC recipients.</value>
        public IList<EmailAddress> BccRecipients
        {
            get
            {
                EnsureAddressProperties();
                return _BccRecipients;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether a read receipt has been requested for this instance
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if a read receipt is requested for this instance; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsReadReceiptRequested
        {
            get { return GetNullablePropertyValue<bool>(NamedProperties.Message.ReadReceiptRequested); }
            set { SetNullablePropertyValue<bool>(NamedProperties.Message.ReadReceiptRequested, value); }
        }
        /// <summary>
        /// Gets or sets a value indicating whether a delivery receipt has been requested for this instance.
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if a delivery receipt has been requested for this instance; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsDeliveryReceiptRequested
        {
            get { return GetNullablePropertyValue<bool>(NamedProperties.Message.DeliveryReceiptRequested); }
            set { SetNullablePropertyValue<bool>(NamedProperties.Message.DeliveryReceiptRequested, value); }

        }
        /// <summary>
        /// Gets the index of the conversation.
        /// </summary>
        /// <value>The index of the conversation.</value>
        public byte[] ConversationIndex
        {
            get { return GetPropertyValue<byte[]>(NamedProperties.Message.ConversationIndex); }
        }

        /// <summary>
        /// Gets or sets the conversation topic.
        /// </summary>
        /// <value>The conversation topic.</value>
        public string ConversationTopic
        {
            get { return GetPropertyValue<string>(NamedProperties.Message.ConversationTopic); }
            set { SetPropertyValue<string>(NamedProperties.Message.ConversationTopic, value); }
        }
        /// <summary>
        /// Gets from email address.
        /// </summary>
        /// <value>The From email address.</value>
        public EmailAddress From
        {
            get
            {
                EnsureAddressProperties();
                return _FromAddress;
            }
        }

        /// <summary>
        /// Gets or sets the internet message id.
        /// </summary>
        /// <value>The internet message id.</value>
        public string InternetMessageId
        {
            get { return GetPropertyValue<string>(NamedProperties.Message.InternetMessageId); }
            set { SetPropertyValue<string>(NamedProperties.Message.InternetMessageId, value); }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is read.
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if this instance is read; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsRead
        {
            get { return CheckFlagValue(GetNullablePropertyValue<int>(NamedProperties.Item.MessageFlags), (int)MessageFlags.IsRead); }
        }
        /// <summary>
        /// Gets or sets a value indicating whether a response is requested for this instance.
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if this instance is response requested; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsResponseRequested
        {
            get { return GetNullablePropertyValue<bool>(NamedProperties.Message.ResponseRequested); }
            set { SetNullablePropertyValue<bool>(NamedProperties.Message.ResponseRequested, value); }
        }
        /// <summary>
        /// Gets or sets the references.
        /// </summary>
        /// <value>The references.</value>
        public string References
        {
            get { return GetPropertyValue<string>(NamedProperties.Message.References); }
            set { SetPropertyValue<string>(NamedProperties.Message.References, value); }
        }

        /// <summary>
        /// Gets the reply to address.
        /// </summary>
        /// <value>The reply to address.</value>
        public EmailAddress ReplyTo
        {
            get
            {
                EnsureAddressProperties();
                return _ReplyToAddress;
            }
        }

        /// <summary>
        /// Gets the spam confidence level.
        /// </summary>
        /// <value>The spam confidence level.</value>
        public int? SpamConfidenceLevel
        {
            get { return GetNullablePropertyValue<int>(NamedProperties.Message.SpamConfidenceLevel); }
        }


        /// <summary>
        /// Initializes the <see cref="Message"/> class.
        /// </summary>
        static Message()
        {
            _RequiredFields = ExtractFieldValues(typeof(NamedProperties.Message));
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Message"/> class.
        /// </summary>
        /// <param name="path">The path of the message.</param>
        /// <param name="connection">The connection.</param>
        public Message(string path, Connection connection)
            : base(path, connection)
        {
            AddRequiredFields(_RequiredFields);
            SetContentClass("IPM.Note", "urn:content-classes:message");
        }

    }
}

==> InfiniTec.Exchange_0.97_Source.zip: WebDav/Message.cs <==
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.CompilerServices;
using System.Net.Mail;
using System.Text.RegularExpressions;
using InfiniTec.Exchange;

namespace InfiniTec.Exchange.WebDav
{
    /// <summary>
    /// This class provides access to commonly used properties used on messages.
    /// </summary>
    [SchemaClass(typeof(NamedProperties.Message))]
    public class MessageItem : Item
    {
        private static string[] _RequiredFields;

        private ChangeNotifyCollection<EmailAddress> _ToRecipients;
        private ChangeNotifyCollection<EmailAddress> _CcRecipients;
        private ChangeNotifyCollection<EmailAddress> _BccRecipients;
        private EmailAddress _SenderAddress;
        private EmailAddress _FromAddress;
        private EmailAddress _ReplyToAddress;
        private bool _PropertiesLoaded;

        /// <summary>
        /// This method is called before the <see cref="Item.Refresh"/> or <see cref="Item.RefreshAsync"/>
        /// operation is called.
        /// </summary>
        protected override void PreRefresh()
        {
            _ToRecipients = null;
            _CcRecipients = null;
            _BccRecipients = null;
            _SenderAddress = null;
            _FromAddress = null;
            _ReplyToAddress = null;
            _PropertiesLoaded = false;

            base.PreRefresh();
        }

        /// <summary>
        /// This method is called before the <see cref="Item.Save"/> or <see cref="Item.SaveAsync"/>
        /// operation is called.
        /// </summary>
        protected override void PreSave()
        {
            CommitAddressList(_ToRecipients, NamedProperties.Message.To);
            CommitAddressList(_CcRecipients, NamedProperties.Message.Cc);
            CommitAddressList(_BccRecipients, NamedProperties.Message.Bcc);

            CommitAddress(_FromAddress, NamedProperties.Message.From);
            CommitAddress(_SenderAddress, NamedProperties.Message.Sender);
            CommitAddress(_ReplyToAddress, NamedProperties.Message.ReplyTo);

            base.PreSave();
        }

        private void CommitAddress(EmailAddress address, string propertyName)
        {
            EmailAddress currentValue;

            LoadAddress(out currentValue, propertyName);
            if (currentValue == address) return;

            if (address == null && BaseItem.Properties.Contains(propertyName))
            {
                BaseItem.Properties.Remove(propertyName);
            }
            else if (address != null)
            {
                SetPropertyValue(propertyName, address.ToString());
            }
        }

        [MethodImpl(MethodImplOptions.Synchronized)]
        private void EnsureAddressProperties()
        {
            if (_PropertiesLoaded) return;

            LoadAddressList(out _ToRecipients, NamedProperties.Message.To);
            LoadAddressList(out _CcRecipients, NamedProperties.Message.Cc);
            LoadAddressList(out _BccRecipients, NamedProperties.Message.Bcc);

            LoadAddress(out _FromAddress, NamedProperties.Message.From);
            LoadAddress(out _SenderAddress, NamedProperties.Message.Sender);
            LoadAddress(out _ReplyToAddress, NamedProperties.Message.ReplyTo);

            _PropertiesLoaded = true;
        }

        /// <summary>
        /// Gets the sender.
        /// </summary>
        /// <value>The sender.</value>
        public EmailAddress Sender
        {
            get
            {
                EnsureAddressProperties();
                return _SenderAddress;
            }

        }
        /// <summary>
        /// Gets the direct recipients.
        /// </summary>
        /// <value>The direct recipients.</value>
        public IList<EmailAddress> ToRecipients
        {
            get
            {
                EnsureAddressProperties();
                return _ToRecipients;
            }
        }
        /// <summary>
        /// Gets the CC recipients.
        /// </summary>
        /// <value>The CC recipients.</value>
        public IList<EmailAddress> CcRecipients
        {
            get
            {
                EnsureAddressProperties();
                return _CcRecipients;
            }
        }
        /// <summary>
        /// Gets the BCC recipients.
        /// </summary>
        /// <value>The BCC recipients.</value>
        public IList<EmailAddress> BccRecipients
        {
            get
            {
                EnsureAddressProperties();
                return _BccRecipients;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether a read receipt has been requested for this instance
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if a read receipt is requested for this instance; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsReadReceiptRequested
        {
            get { return GetNullablePropertyValue<bool>(NamedProperties.Message.ReadReceiptRequested); }
            set { SetNullablePropertyValue(NamedProperties.Message.ReadReceiptRequested, value); }
        }
        /// <summary>
        /// Gets or sets a value indicating whether a delivery receipt has been requested for this instance.
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if a delivery receipt has been requested for this instance; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsDeliveryReceiptRequested
        {
            get { return GetNullablePropertyValue<bool>(NamedProperties.Message.DeliveryReceiptRequested); }
            set { SetNullablePropertyValue(NamedProperties.Message.DeliveryReceiptRequested, value); }

        }
        /// <summary>
        /// Gets the index of the conversation.
        /// </summary>
        /// <value>The index of the conversation.</value>
        public byte[] ConversationIndex
        {
            get { return GetPropertyValue<byte[]>(NamedProperties.Message.ConversationIndex); }
        }

        /// <summary>
        /// Gets or sets the conversation topic.
        /// </summary>
        /// <value>The conversation topic.</value>
        public string ConversationTopic
        {
            get { return GetPropertyValue<string>(NamedProperties.Message.ConversationTopic); }
            set { SetPropertyValue(NamedProperties.Message.ConversationTopic, value); }
        }
        /// <summary>
        /// Gets from email address.
        /// </summary>
        /// <value>The From email address.</value>
        public EmailAddress From
        {
            get
            {
                EnsureAddressProperties();
                return _FromAddress;
            }
        }

        /// <summary>
        /// Gets or sets the internet message id.
        /// </summary>
        /// <value>The internet message id.</value>
        public string InternetMessageId
        {
            get { return GetPropertyValue<string>(NamedProperties.Message.InternetMessageId); }
            set { SetPropertyValue(NamedProperties.Message.InternetMessageId, value); }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is read.
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if this instance is read; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsRead
        {
            get { return CheckFlagValue(GetNullablePropertyValue<int>(NamedProperties.Item.MessageFlags), (int)MessageFlags.IsRead); }
        }
        /// <summary>
        /// Gets or sets a value indicating whether a response is requested for this instance.
        /// </summary>
        /// <value>
        /// 	<see langword="true"/> if this instance is response requested; otherwise, <see langword="false"/>.
        /// </value>
        public bool? IsResponseRequested
        {
            get { return GetNullablePropertyValue<bool>(NamedProperties.Message.ResponseRequested); }
            set { SetNullablePropertyValue(NamedProperties.Message.ResponseRequested, value); }
        }
        /// <summary>
        /// Gets or sets the references.
        /// </summary>
        /// <value>The references.</value>
        public string References
        {
            get { return GetPropertyValue<string>(NamedProperties.Message.References); }
            set { SetPropertyValue(NamedProperties.Message.References, value); }
        }

        /// <summary>
        /// Gets the reply to address.
        /// </summary>
        /// <value>The reply to address.</value>
        public EmailAddress ReplyTo
        {
            get
            {
                EnsureAddressProperties();
                return _ReplyToAddress;
            }
        }

        /// <summary>
        /// Gets the spam confidence level.
        /// </summary>
        /// <value>The spam confidence level.</value>
        public int? SpamConfidenceLevel
        {
            get { return GetNullablePropertyValue<int>(NamedProperties.Message.SpamConfidenceLevel); }
        }


        /// <summary>
        /// Initializes the <see cref="NamedProperties.Message"/> class.
        /// </summary>
        static MessageItem()
        {
            _RequiredFields = SchemaHelper.ExtractFieldValues(typeof(NamedProperties.Message));
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MessageItem"/> class.
        /// </summary>
        public MessageItem()
        {
            Initialize();
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="NamedProperties.Message"/> class.
        /// </summary>
        /// <param name="path">The path of the message.</param>
        /// <param name="connection">The connection.</param>
        public MessageItem(string path, Connection connection)
            : base(path, connection)
        {
            Initialize();
        }

        private void Initialize()
        {
            AddRequiredFields(_RequiredFields);
            SetContentClass("IPM.Note", "urn:content-classes:message");
        }

    }
}
